<?php
require_once 'include.php';

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
if (!defined('_PS_VERSION_'))
	exit;

class RbsPayment extends PaymentModule
{
	public $default_order_state;

	const _REGISTERED_BUT_NOT_PAID_FOR_                          = 0;
	const _PRE_AUTHORIZATION_                                    = 1;
	const _AUTHORIZATION_                                        = 2;
	const _AUTHORIZATION_CANCELED_                               = 3;
	const _ORDER_RETURN_                                         = 4;
	const _ACS_INITIATED_AUTHORIZATION_THROUGH_THE_ISSUING_BANK_ = 5;
	const _AUTHORIZATION_FAILED_                                 = 6;

	static $cache = array();

	protected $login;
	protected $password;

    protected $real_link = PROD_URL;
    protected $debug_link = TEST_URL;

	public $debug;
	public $ext = '.do';


	public function __construct()
	{
		$this->name            = 'rbspayment';
		$this->tab             = 'payments_gateways';
		$this->version         = '1.0.4';
		$this->author          = 'RBS';
		$this->controllers = array('payment', 'validation');
		$this->currencies      = true;
		$this->currencies_mode = 'checkbox';
		$this->bootstrap = true;

		parent::__construct();

		$this->displayName = 'Оплата картой';
		$this->description = 'Модуль оплаты картой через платежный шлюз';
		$this->confirmUninstall = 'Вы действительно хотите удалить модуль?';
		$this->default_order_state = Configuration::get('PS_RBS_STATE_DEFAULT');
		$this->login       = Configuration::get('PS_RBS_LOGIN');
		$this->password    = Configuration::get('PS_RBS_PASSWORD');
		$this->debug       = Configuration::get('PS_RBS_DEBUG');
	}

	public function install()
	{
		$sql = 'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'rbs_payment` (
                  `id_order` int(11) NOT NULL,
                  `order_number` text COLLATE utf8_unicode_ci NOT NULL,
                  `rbs_order_id` text COLLATE utf8_unicode_ci NOT NULL,
                  `form_url` text COLLATE utf8_unicode_ci NOT NULL,
                  PRIMARY KEY (`id_order`),
                  UNIQUE KEY `id_order` (`id_order`)
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;';

		if (!parent::install()
			|| !$this->registerHook('payment')
			|| !$this->registerHook('paymentReturn')
			|| !$this->registerHook('paymentOptions')
			|| !Db::getInstance()->execute($sql)
            )
			return false;
		return true;
	}

	public function uninstall()
	{
		$sql = 'DROP TABLE IF EXISTS `'._DB_PREFIX_.'rbs_payment`';
		if (!parent::uninstall()
            || !Db::getInstance()->execute($sql)
        )
			return false;
		return true;
	}

	public function hookPayment()
	{
		if (!$this->active)
			return;

		$this->context->controller->addCSS($this->_path.'views/css/rbspayment.css', 'all');

		$this->smarty->assign(array(
			'module'        => $this,
			'this_path'     => $this->_path,
			'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		));
		return $this->display(__FILE__, 'payment.tpl');
	}

	public function hookDisplayPaymentReturn()
	{
		$this->hookPayment();
	}

	public function hookPaymentOptions($params)
	{
		if (!$this->active) {
			return;
		}

		if (!$this->checkCurrency($params['cart'])) {
			return;
		}

		$this->smarty->assign(array(
			'module'        => $this,
			'this_path'     => $this->_path,
			'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		));

		$newOption = new PaymentOption();
		$newOption->setCallToActionText("Оплата картой банка")
			->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
			->setAdditionalInformation($this->fetch('module:rbspayment/views/templates/hook/payment.tpl'));
		$payment_options = [
			$newOption,
		];

		return $payment_options;
	}

	public function gateway($method, $data, $url, $log = true)
	{
		$curl = curl_init();
		curl_setopt_array($curl, array(
			CURLOPT_URL => $url.$method,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => http_build_query($data)
		));
		$response = curl_exec($curl);
		$response = json_decode($response, true);

		if ($log == true)
			$this->logger('Request: '.$url.$method.': '.print_r($data, true).'Response: '. print_r($response, true));

		curl_close($curl);

		return $response;
	}

	public function logger($message)
	{
		$logger = new FileLogger();
		$dir = version_compare(_PS_VERSION_, '1.7', '<') ? '/log/' : '/app/logs/';
		$logger->setFilename(_PS_ROOT_DIR_.$dir.date('Y-m-d').'_rbspayment.log');
		$logger->logInfo(date('H:i:s')."\n".$message);
	}

	public function getPaymentGateValues($id_order)
	{
		$cache_key = 'getPaymentGateValues_'.$id_order;

		if (!isset(self::$cache[$cache_key]))
		{

			$result = Db::getInstance()->getRow('SELECT * FROM `'._DB_PREFIX_.'rbs_payment` WHERE `id_order` = \''.(int)$id_order.'\'');

			if ($result)
			{
				$curl_data = array('userName' => $this->getLogin(), 'password' => $this->getPassword(), 'orderId' => $result['rbs_order_id']);
				$return_gate = $this->gateway('getOrderStatus'.$this->ext, $curl_data, $this->getLink(), $this->debug);

				$return = $this->noCamelCase($return_gate);

				if (isset($return['orderstatus']))
				{
					$result['order_status']  = $return['orderstatus'];
					self::$cache[$cache_key] = $result;
				}
			}
			else
				self::$cache[$cache_key] = false;
		}
		return self::$cache[$cache_key];
	}

	public function registerInPaymentGate($id_order)
	{
		$cache_key = 'getPaymentGateValues_'.$id_order;

		if (isset(self::$cache[$cache_key]))
			unset(self::$cache[$cache_key]);

		$order        = new Order((int)$id_order);
		$order_number = Db::getInstance()->getValue('SELECT IFNULL(`order_number`, 0) FROM `'
				._DB_PREFIX_.'rbs_payment` WHERE `id_order` = \''.(int)$order->id.'\'') + 1;

		$curl_data = array('userName' => $this->getLogin(),
			'password' => $this->getPassword(),
			'amount' => (int)$order->total_paid * 100,
			'orderNumber' => (int)$order->id.'_'.$order_number,
			'description' => '',
			'returnUrl' => $this->context->link->getModuleLink($this->name, 'OrderState'),
			'failUrl' => $this->context->link->getModuleLink($this->name, 'OrderState'),
		);

		$result_gate = $this->gateway('register'.$this->ext, $curl_data, $this->getLink(), $this->debug);

		$result = $this->noCamelCase($result_gate);

		if (!isset($result['errorcode']) || !$result['errorcode'])
		{

			$sql = 'INSERT INTO `'._DB_PREFIX_.'rbs_payment` (`id_order`, `order_number`, `rbs_order_id`, `form_url`)
                        VALUES (\''.(int)$order->id.'\', \''.pSQL($order_number).'\' , \''.pSQL($result['orderid']).'\', \''.pSQL($result['formurl']).'\')
                        ON DUPLICATE KEY UPDATE
                            `order_number` = \''.pSQL($order_number).'\',
                            `rbs_order_id` = \''.(int)$result['orderid'].'\',
                            `form_url` = \''.pSQL($result['formurl']).'\'';
			return Db::getInstance()->execute($sql);

		}
		return false;
	}

	/**
	 * @return string
	 */
	public function getContentWrap()
	{
		$this->context->controller->addCSS($this->_path.'views/css/rbspayment.css', 'all');
		if (Tools::isSubmit('saveSettings'))
		{
			$this->login    = Tools::getValue('login');
			$this->password = Tools::getValue('password');
			$this->debug    = Tools::getValue('debug');
			Configuration::updateValue('PS_RBS_LOGIN', $this->login);
			Configuration::updateValue('PS_RBS_PASSWORD', $this->password);
			Configuration::updateValue('PS_RBS_DEBUG', $this->debug);
			Configuration::updateValue('PS_RBS_STATE', Tools::getValue('authorization_state'));
			Configuration::updateValue('PS_RBS_STATE_DEFAULT', Tools::getValue('default_state'));
			Configuration::updateValue('PS_RBS_STATE_ERROR', Tools::getValue('error_state'));
		}

		$query = 'SELECT `id_order_state`, `name` FROM `'._DB_PREFIX_.'order_state_lang` WHERE `id_lang` = "'.(int)$this->context->language->id.'"';
		$order_state = DB::getInstance()->executeS($query);

		return $this->adminForm($order_state);
	}

	public function adminForm($order_state)
	{
		$fields_form = array(
			array(
				'label' => 'Логин',
				'hint' => 'Логин платежной системы, выданный банком',
				'required' => true,
				'type' => 'text',
				'name' => 'login',
				'class' => 'fixed-width-lg'
			),
			array(
				'label' => 'Пароль',
				'hint' => 'Пароль платежной системы, выданный банком',
				'required' => true,
				'type' => 'text',
				'name' => 'password',
                'class' => 'fixed-width-lg'
			),
			array(
				'label' => 'Статус по умолчанию',
				'hint' => 'Статус заказа по умолчанию, когда выбран текущий метод оплаты',
				'type' => 'select',
				'name' => 'default_state',
				'options' => array(
					'id' => 'id_order_state',
					'name' => 'name',
					'query' => $order_state
					),
				),
			array(
				'label' => 'Статус авторизации',
				'hint' => 'Статус заказа после успешной авторизации и оплаты',
				'type' => 'select',
				'name' => 'authorization_state',
				'options' => array(
					'id' => 'id_order_state',
					'name' => 'name',
					'query' => $order_state
					),
				),
				array(
					'label' => 'Статус ошибки',
					'hint' => 'Статус заказа при ошибке оплаты',
					'type' => 'select',
					'name' => 'error_state',
					'options' => array(
						'id' => 'id_order_state',
						'name' => 'name',
						'query' => $order_state
						),
				),
				array(
					'label' => 'Режим отладки',
                    'hint' => 'Будет использоваться тестовый шлюз',
					'type' => 'switch',
					'name' => 'debug',
					'values' => array(
						array(
							'value' => 1
						),
						array(
							'value' => 0
						)
					),
				)
		);

		$fields_value = array(
			'login' => Configuration::get('PS_RBS_LOGIN') ? Configuration::get('PS_RBS_LOGIN') : null,
			'password' => Configuration::get('PS_RBS_PASSWORD') ? Configuration::get('PS_RBS_PASSWORD') : null,
			'default_state' => Configuration::get('PS_RBS_STATE_DEFAULT') ? Configuration::get('PS_RBS_STATE_DEFAULT') : null,
			'authorization_state' => Configuration::get('PS_RBS_STATE') ? Configuration::get('PS_RBS_STATE') : null,
			'error_state' => Configuration::get('PS_RBS_STATE_ERROR') ? Configuration::get('PS_RBS_STATE_ERROR') : null,
			'debug' => Configuration::get('PS_RBS_DEBUG') ? Configuration::get('PS_RBS_DEBUG') : '0'
		);

		$fields = array(
			'setting_fields' => array(
				'form' => array(
					'legend' => array(
						'title' => 'Настройки плагина'
					),
					'input' => $fields_form,
					'submit' => array(
						'title' => $this->l('Save'),
						'name' => 'saveSettings',
						'class' => 'button btn btn-default pull-right',
						'desc' => ''
					)
				)
			)
		);

		$helper_form = new HelperForm();
		$helper_form->fields_value = $fields_value;
		$helper_form->token = Tools::getValue('token');
		$helper_form->currentIndex = 'index.php?controller=AdminModules&configure='.$this->name
			.'&tab_module=front_office_features&module_name='.$this->name;

		return $helper_form->generateForm($fields);
	}

	public function getAdditionalActions($params)
	{
		if ($params['id_order_state'] != Configuration::get('PS_RBS_STATE'))
			if ($params['id_order_state'] != Configuration::get('PS_RBS_STATE_DEFAULT'))
					return;

		$info = $this->getPaymentGateValues($params['id_order']);

		$this->context->smarty->assign(array(
			'payment_link' => $this->context->link->getModuleLink($this->name, 'PaymentGate'),
			'id_order'     => $params['id_order'],
			'order_status' => $info['order_status']
		));
		return $this->display(__FILE__, 'views/templates/front/additional_action.tpl');
	}

	public function getLink()
	{
		return ((int)$this->debug) ? $this->debug_link : $this->real_link;
	}

	public function getLogin()
	{
		return $this->login;
	}

	public function getPassword()
	{
		return $this->password;
	}

	public function getRedirectLink()
	{
		return $this->context->link->getModuleLink($this->name, 'PaymentGate', array(
			'id_order' => $this->currentOrder
		));
	}

	public function checkCurrency($cart)
	{
		$currency_order = new Currency($cart->id_currency);
		$currencies_module = $this->getCurrency($cart->id_currency);

		if (is_array($currencies_module))
			foreach ($currencies_module as $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
		return false;
	}

	public function noCamelCase($obj)
	{
		$return = array();
		if (is_object($obj) || is_array($obj))
		{
			foreach ($obj as $key => $value)
			{
				$k = Tools::strtolower($key);
				$return[$k] = $value;
			}
			return $return;
		}
		return false;
	}

	public function getContent()
	{
		$this->registerSmartyFunctions();
		$this->context->controller->addJS('http://#'
			.Context::getContext()->language->iso_code
		);
		$this->context->smarty->assign(array(
			'content_tab' => $this->getContentWrap(),
//			'documentation' => $this->getDocumentation()
		));

		return $this->context->smarty->fetch(_PS_MODULE_DIR_.$this->name.'/views/templates/admin/content.tpl');
	}


	/**
	 * @param string $pattern
	 * @param int $flags
	 * @return array
	 */
	public static function globRecursive($pattern, $flags = 0)
	{
		$files = glob($pattern, $flags);
		if (!$files)
			$files = array();

		foreach (glob(dirname($pattern).'/*', GLOB_ONLYDIR | GLOB_NOSORT) as $dir)
			/** @noinspection SlowArrayOperationsInLoopInspection */
			$files = array_merge($files, self::globRecursive($dir.'/'.basename($pattern), $flags));

		return $files;
	}

	public static function noEscape($value)
	{
		return $value;
	}
	public function registerSmartyFunctions()
	{
		$smarty = $this->context->smarty;
		if (!array_key_exists('get_image_lang', $smarty->registered_plugins['function']))
			smartyRegisterFunction($smarty, 'function', 'get_image_lang', array($this, 'getImageLang'));
		if (!array_key_exists('no_escape', $smarty->registered_plugins['modifier']))
			smartyRegisterFunction($smarty, 'modifier', 'no_escape', array(__CLASS__, 'noEscape'));
		if (class_exists('TransModRbs'))
		{
			if (!array_key_exists('ld', $smarty->registered_plugins['modifier']))
				smartyRegisterFunction($smarty, 'modifier', 'ld', array(TransModRbs::getInstance(), 'ld'));
		}
	}

	public function getImageLang($smarty)
	{
		if (_PS_VERSION_ < 1.5)
			$cookie = &$GLOBALS['cookie'];
		else
		{
			$cookie = $this->context->cookie;
			$cookie->id_lang = $this->context->language->id;
		}

		$path = $smarty['path'];
		$module_path = $this->name.'/views/img/';
		$current_language = new Language($cookie->id_lang);
		$module_lang_path = $module_path.$current_language->iso_code.'/';
		$module_lang_default_path = $module_path.'en/';
		$path_image = false;
		if (file_exists(_PS_MODULE_DIR_.$module_lang_path.$path))
			$path_image = _MODULE_DIR_.$module_lang_path.$path;
		elseif (file_exists(_PS_MODULE_DIR_.$module_lang_default_path.$path))
			$path_image = _MODULE_DIR_.$module_lang_default_path.$path;

		if ($path_image)
			return '<img class="thumbnail" src="'.$path_image.'">';
		else
			return '[can not load image "'.$path.'"]';
	}
}